@@ -2,6 +2,8 @@ import warnings
 from operator import attrgetter
 from typing import (
     TYPE_CHECKING,
+    Any,
+    Dict,
     Iterable,
     Iterator,
     List,
@@ -15,11 +17,13 @@ import numpy as np
 import pandas as pd
 from shapely.geometry import LineString, MultiLineString, Point, Polygon
 
-from ..core.geodesy import destination, mrr_diagonal
+from ..core.geodesy import destination, distance, mrr_diagonal
 from ..core.iterator import flight_iterator
 from ..core.time import deltalike, to_timedelta
 
 if TYPE_CHECKING:
+    from cartes.osm import Overpass  # noqa: 401
+
     from ..core import Flight, FlightPlan  # noqa: 401
     from ..core.mixins import PointMixin  # noqa: 401
     from ..core.structure import Airport, Navaid  # noqa: 401
@@ -84,7 +88,7 @@ class NavigationFeatures:
             airport if isinstance(airport, Airport) else airports[airport]
         )
 
-    def takeoff_airport(self, **kwargs) -> "Airport":
+    def takeoff_airport(self, **kwargs: Any) -> "Airport":
         """Returns the most probable takeoff airport based on the first location
         in the trajectory.
 
@@ -125,7 +129,7 @@ class NavigationFeatures:
             airport if isinstance(airport, Airport) else airports[airport]
         )
 
-    def landing_airport(self, **kwargs) -> "Airport":
+    def landing_airport(self, **kwargs: Any) -> "Airport":
         """Returns the most probable landing airport based on the last location
         in the trajectory.
 
@@ -177,7 +181,11 @@ class NavigationFeatures:
         self = cast("Flight", self)
 
         _airport = airports[airport] if isinstance(airport, str) else airport
-        if _airport is None or _airport.runways.shape.is_empty:
+        if (
+            _airport is None
+            or _airport.runways is None
+            or _airport.runways.shape.is_empty
+        ):
             return None
 
         if isinstance(_airport.runways.shape, LineString):
@@ -498,7 +506,11 @@ class NavigationFeatures:
         self = cast("Flight", self).phases()
 
         _airport = airports[airport] if isinstance(airport, str) else airport
-        if _airport is None or _airport.runways.shape.is_empty:
+        if (
+            _airport is None
+            or _airport.runways is None
+            or _airport.runways.shape.is_empty
+        ):
             return None
 
         nb_run = len(_airport.runways.data)
@@ -728,10 +740,15 @@ class NavigationFeatures:
             future.
 
         """
-        candidate = self.query("altitude < 8000")  # type: ignore
+        # The following cast secures the typing
+        self = cast("Flight", self)
+
+        candidate = self.query("altitude < 8000")
         if candidate is not None:
             for chunk in candidate.split("10T"):
                 point = chunk.query("altitude == altitude.min()")
+                if point is None:
+                    return
                 if dataset is None:
                     cd = point.landing_airport()
                 else:
@@ -749,9 +766,12 @@ class NavigationFeatures:
         """
         from ..data import airports
 
-        f_above = self.query("altitude > 15000")  # type: ignore
+        # The following cast secures the typing
+        self = cast("Flight", self)
+
+        f_above: Optional["Flight"] = self.query("altitude > 15000")
         if (
-            self.destination != self.destination  # type: ignore
+            self.destination != self.destination
             or airports[self.destination] is None  # type: ignore
             or f_above is None
         ):
@@ -773,7 +793,11 @@ class NavigationFeatures:
     @property
     def holes(self) -> int:
         """Returns the number of 'holes' in a trajectory."""
-        simplified: "Flight" = self.simplify(25)  # type: ignore
+
+        # The following cast secures the typing
+        self = cast("Flight", self)
+
+        simplified = self.simplify(25)
         if simplified.shape is None:
             return -1
         return len(simplified.shape.buffer(1e-3).interiors)
@@ -781,11 +805,11 @@ class NavigationFeatures:
     @flight_iterator
     def holding_pattern(
         self,
-        min_altitude=7000,
-        turning_threshold=0.5,
-        low_limit=pd.Timedelta("30 seconds"),  # noqa: B008
-        high_limit=pd.Timedelta("10 minutes"),  # noqa: B008
-        turning_limit=pd.Timedelta("5 minutes"),  # noqa: B008
+        min_altitude: float = 7000,
+        turning_threshold: float = 0.5,
+        low_limit: pd.Timedelta = pd.Timedelta("30 seconds"),  # noqa: B008
+        high_limit: pd.Timedelta = pd.Timedelta("10 minutes"),  # noqa: B008
+        turning_limit: pd.Timedelta = pd.Timedelta("5 minutes"),  # noqa: B008
     ) -> Iterator["Flight"]:
         """Iterates on parallel segments candidates for identifying
         a holding pattern.
@@ -863,7 +887,11 @@ class NavigationFeatures:
         self = cast("Flight", self)
 
         _airport = airports[airport] if isinstance(airport, str) else airport
-        if _airport is None or _airport.runways.shape.is_empty:
+        if (
+            _airport is None
+            or _airport.runways is None
+            or _airport.runways.shape.is_empty
+        ):
             return None
 
         segment = self.filter().inside_bbox(_airport)
@@ -883,7 +911,7 @@ class NavigationFeatures:
         self,
         speed_threshold: float = 2,
         time_threshold: str = "30s",
-        filter_dict=dict(compute_gs=3),
+        filter_dict: Dict[str, int] = dict(compute_gs=3),  # noqa: B006
         resample_rule: str = "5s",
     ) -> Optional["Flight"]:
         """
@@ -905,7 +933,7 @@ class NavigationFeatures:
 
         moving = (
             resampled.cumulative_distance()
-            .filter(**filter_dict)
+            .filter(**filter_dict)  # type: ignore
             .query(f"compute_gs > {speed_threshold}")
         )
         if moving is None:
@@ -926,7 +954,7 @@ class NavigationFeatures:
     def pushback(
         self,
         airport: Union[str, "Airport"],
-        filter_dict=dict(
+        filter_dict: Dict[str, int] = dict(
             compute_track_unwrapped=21, compute_track=21, compute_gs=21
         ),
         track_threshold: float = 90,
@@ -952,7 +980,11 @@ class NavigationFeatures:
         self = cast("Flight", self)
 
         _airport = airports[airport] if isinstance(airport, str) else airport
-        if _airport is None or _airport.runways.shape.is_empty:
+        if (
+            _airport is None
+            or _airport.runways is None
+            or _airport.runways.shape.is_empty
+        ):
             return None
 
         within_airport = self.inside_bbox(_airport)
@@ -966,30 +998,31 @@ class NavigationFeatures:
         after_parking = within_airport.after(parking_position.start)
         assert after_parking is not None
 
-        in_movement = after_parking.moving()
+        in_movement = cast(Optional["Flight"], after_parking.moving())
 
         if in_movement is None:
             return None
 
-        direction_change = (
+        direction_change = cast(
+            Optional["Flight"],
             # trim the first few seconds to avoid annoying first spike
-            in_movement.first("5T")
+            in_movement.first("5T")  # type:ignore
             .last("4T30s")
             .cumulative_distance()
             .unwrap(["compute_track"])
-            .filter(**filter_dict)
+            .filter(**filter_dict)  # type: ignore
             .diff("compute_track_unwrapped")
-            .query(f"compute_track_unwrapped_diff.abs() > {track_threshold}")
+            .query(f"compute_track_unwrapped_diff.abs() > {track_threshold}"),
         )
 
         if direction_change is None:
             return None
 
-        return in_movement.before(direction_change.start).assign(
-            parking_position=parking_position.parking_position_max
-        )
+        return in_movement.before(  # type: ignore
+            direction_change.start
+        ).assign(parking_position=parking_position.parking_position_max)
 
-    def is_from_inertial(self, freq_threshold=0.05) -> bool:
+    def is_from_inertial(self, freq_threshold: float = 0.05) -> bool:
         """
         Returns True if ground trajectory data looks noisy.
 
@@ -1012,7 +1045,9 @@ class NavigationFeatures:
         )
         if 90 not in freq.index or -90 not in freq.index:
             return False
-        return freq[90] > freq_threshold and freq[-90] > freq_threshold
+        return (  # type: ignore
+            freq[90] > freq_threshold and freq[-90] > freq_threshold
+        )
 
     @flight_iterator
     def slow_taxi(
@@ -1105,3 +1140,135 @@ class NavigationFeatures:
             candidate = self.between(segment_times[0], segment_times[-1])
             if candidate is not None:
                 yield candidate
+
+    @flight_iterator
+    def ground_trajectory(
+        self, airport: Union[str, "Airport"]
+    ) -> Iterator["Flight"]:
+        """Returns the ground part of the trajectory limited to the apron
+        of the airport passed in parameter.
+
+        The same trajectory could use the apron several times, hence the safest
+        option to return a FlightIterator.
+        """
+
+        from traffic.data import airports
+
+        self = cast("Flight", self)
+        airport_ = airports[airport] if isinstance(airport, str) else airport
+        assert airport_ is not None
+
+        has_onground = "onground" in self.data.columns
+        criterion = "altitude < 5000"
+        if has_onground:
+            criterion += " or onground"
+
+        low_altitude = self.query(criterion)
+        if low_altitude is None:
+            return
+        if airport_.shape is None:
+            raise ValueError("No shape available for the given airport")
+        for low_segment in low_altitude.split("10T"):
+            for airport_segment in low_segment.clip_iterate(
+                airport_.shape.buffer(5e-3)
+            ):
+                if has_onground:
+                    onground = airport_segment.query("onground")
+                else:
+                    onground = airport_segment.query(
+                        "altitude < 500 or altitude != altitude"
+                    )
+                if onground is not None:
+                    yield onground
+
+    @flight_iterator
+    def on_taxiway(
+        self,
+        airport_or_taxiways: Union[str, pd.DataFrame, "Airport", "Overpass"],
+        *,
+        tolerance: float = 15,
+        max_dist: float = 85,
+    ) -> Iterator["Flight"]:
+        """
+        Iterates on segments of trajectory matching a single runway label.
+        """
+
+        from ..core.structure import Airport
+        from ..data import airports
+
+        self = cast("Flight", self)
+        if isinstance(airport_or_taxiways, str):
+            airport_or_taxiways = airports[airport_or_taxiways]
+        # This is obvious, but not for MyPy
+        assert not isinstance(airport_or_taxiways, str)
+
+        taxiways_ = (
+            airport_or_taxiways.taxiway
+            if isinstance(airport_or_taxiways, Airport)
+            else airport_or_taxiways
+        )
+
+        # decompose with a function because MyPy is lost
+        def taxi_df(taxiways_: Union["Overpass", pd.DataFrame]) -> pd.DataFrame:
+            if isinstance(taxiways_, pd.DataFrame):
+                return taxiways_
+            if taxiways_.data is None:
+                raise ValueError("No taxiway information")
+            return taxiways_.data
+
+        taxiways = (  # one entry per runway label
+            taxi_df(taxiways_)
+            .groupby("ref")
+            .agg({"geometry": list})["geometry"]
+            .apply(MultiLineString)
+            .to_frame()
+        )
+
+        simplified_df = cast(
+            pd.DataFrame, self.simplify(tolerance=tolerance).data
+        )
+        if simplified_df.shape[0] < 2:
+            return
+
+        previous_candidate = None
+        first = simplified_df.iloc[0]
+        for _, second in simplified_df.iloc[1:].iterrows():
+
+            p1 = Point(first.longitude, first.latitude)
+            p2 = Point(second.longitude, second.latitude)
+
+            def extremities_dist(twy: MultiLineString) -> float:
+                p1_proj = twy.interpolate(twy.project(p1))
+                p2_proj = twy.interpolate(twy.project(p2))
+                d1 = distance(p1_proj.y, p1_proj.x, p1.y, p1.x)
+                d2 = distance(p2_proj.y, p2_proj.x, p2.y, p2.x)
+                return d1 + d2  # type: ignore
+
+            temp_ = taxiways.assign(dist=np.vectorize(extremities_dist))
+            start, stop, ref, dist = (
+                first.timestamp,
+                second.timestamp,
+                temp_.dist.idxmin(),
+                temp_.dist.min(),
+            )
+            if dist < max_dist:
+                candidate = self.assign(taxiway=ref).between(start, stop)
+                if previous_candidate is None:
+                    previous_candidate = candidate
+
+                else:
+                    prev_ref = previous_candidate.taxiway_max
+                    delta = start - previous_candidate.stop
+                    if prev_ref == ref and delta < pd.Timedelta("1T"):
+                        previous_candidate = self.assign(taxiway=ref).between(
+                            previous_candidate.start, stop
+                        )
+
+                    else:
+                        yield previous_candidate
+                        previous_candidate = candidate
+
+            first = second
+
+        if previous_candidate is not None:
+            yield previous_candidate
