@@ -7,7 +7,7 @@ from datetime import timedelta
 from io import StringIO
 from pathlib import Path
 from tempfile import gettempdir
-from typing import Callable, Dict, Iterable, List, Optional, Tuple, Union, cast
+from typing import Any, Dict, Iterable, List, Optional, Tuple, Union, cast
 
 import paramiko
 from tqdm.autonotebook import tqdm
@@ -18,6 +18,7 @@ from shapely.geometry.base import BaseGeometry
 
 from ...core import Flight, Traffic
 from ...core.time import round_time, split_times, timelike, to_datetime
+from ...core.types import ProgressbarType
 from .raw_data import RawData
 
 
@@ -301,6 +302,12 @@ class Impala(object):
     def _format_history(
         df: pd.DataFrame, nautical_units: bool = True
     ) -> pd.DataFrame:
+        """
+        This function can be used in tandem with `_format_dataframe()` to
+        convert (historical data specific) column types and optionally convert
+        the units back to nautical miles, feet and feet/min.
+
+        """
 
         if "lastcontact" in df.columns:
             df = df.drop(["lastcontact"], axis=1)
@@ -358,11 +365,11 @@ class Impala(object):
         request_pattern: str,
         start: timelike,
         stop: timelike,
-        *args,  # more reasonable to be explicit about arguments
+        *args: Any,  # more reasonable to be explicit about arguments
         columns: List[str],
         date_delta: timedelta = timedelta(hours=1),  # noqa: B008
         cached: bool = True,
-        progressbar: Union[bool, Callable[[Iterable], Iterable]] = True,
+        progressbar: Union[bool, ProgressbarType[Any]] = True,
     ) -> pd.DataFrame:
         """Splits and sends a custom request.
 
@@ -403,7 +410,7 @@ class Impala(object):
         if progressbar is False:
             progressbar = iter
 
-        progressbar = cast(Callable[[Iterable], Iterable], progressbar)
+        progressbar = cast(ProgressbarType[Any], progressbar)
 
         cumul: List[pd.DataFrame] = []
         sequence = list(split_times(start, stop, date_delta))
@@ -439,7 +446,7 @@ class Impala(object):
         self,
         start: timelike,
         stop: Optional[timelike] = None,
-        *args,  # more reasonable to be explicit about arguments
+        *args: Any,  # more reasonable to be explicit about arguments
         departure_airport: Union[None, str, Iterable[str]] = None,
         arrival_airport: Union[None, str, Iterable[str]] = None,
         airport: Union[None, str, Iterable[str]] = None,
@@ -447,7 +454,7 @@ class Impala(object):
         icao24: Union[None, str, Iterable[str]] = None,
         cached: bool = True,
         limit: Optional[int] = None,
-        progressbar: Union[bool, Callable[[Iterable], Iterable]] = True,
+        progressbar: Union[bool, ProgressbarType[Any]] = True,
     ) -> pd.DataFrame:
         """Lists flights departing or arriving at a given airport.
 
@@ -530,7 +537,7 @@ class Impala(object):
         if progressbar is False:
             progressbar = iter
 
-        progressbar = cast(Callable[[Iterable], Iterable], progressbar)
+        progressbar = cast(ProgressbarType[Any], progressbar)
 
         other_params = ""
 
@@ -631,7 +638,7 @@ class Impala(object):
         self,
         start: timelike,
         stop: Optional[timelike] = None,
-        *args,  # more reasonable to be explicit about arguments
+        *args: Any,  # more reasonable to be explicit about arguments
         date_delta: timedelta = timedelta(hours=1),  # noqa: B008
         return_flight: bool = False,
         callsign: Union[None, str, Iterable[str]] = None,
@@ -649,7 +656,8 @@ class Impala(object):
         other_tables: str = "",
         other_params: str = "",
         nautical_units: bool = True,
-        progressbar: Union[bool, Callable[[Iterable], Iterable]] = True,
+        time_buffer: Union[None, str, pd.Timedelta] = None,
+        progressbar: Union[bool, ProgressbarType[Any]] = True,
     ) -> Optional[Union[Traffic, Flight]]:
 
         """Get Traffic from the OpenSky Impala shell.
@@ -716,6 +724,11 @@ class Impala(object):
               many sensors received each record;
             - **nautical_units** (boolean, default: True): convert data stored
               in Impala to standard nautical units (ft, ft/min, knots).
+            - **time_buffer** (str or pd.Timedelta, default: None): time buffer
+              used to extend time bounds for flights in the OpenSky flight
+              tables: requests will get flights between ``start - time_buffer``
+              and ``stop + time_buffer``.
+              If no airport is specified, the parameter is ignored.
             - **cached** (boolean, default: True): switch to False to force a
               new request to the database regardless of the cached files;
               delete previous cache files;
@@ -742,11 +755,14 @@ class Impala(object):
         if progressbar is False:
             progressbar = iter
 
-        progressbar = cast(Callable[[Iterable], Iterable], progressbar)
+        progressbar = cast(ProgressbarType[Any], progressbar)
 
         airports_params = [airport, departure_airport, arrival_airport]
         count_airports_params = sum(x is not None for x in airports_params)
 
+        if count is True and serials is None:
+            other_tables += ", state_vectors_data4.serials s "
+
         if isinstance(serials, Iterable):
             other_tables += ", state_vectors_data4.serials s "
             other_params += "and s.ITEM in {} ".format(tuple(serials))
@@ -755,15 +771,11 @@ class Impala(object):
             other_params += "and s.ITEM = {} ".format(serials)
 
         if isinstance(icao24, str):
-            other_params += "and {}icao24='{}' ".format(
-                "sv." if count_airports_params > 0 else "", icao24.lower()
-            )
+            other_params += "and icao24='{}' ".format(icao24.lower())
 
         elif isinstance(icao24, Iterable):
             icao24 = ",".join("'{}'".format(c.lower()) for c in icao24)
-            other_params += "and {}icao24 in ({}) ".format(
-                "sv." if count_airports_params > 0 else "", icao24
-            )
+            other_params += "and icao24 in ({}) ".format(icao24)
 
         if isinstance(callsign, str):
             if (
@@ -772,23 +784,18 @@ class Impala(object):
                 - set(string.digits)
                 - set("%_")
             ):  # if regex like characters
-                other_params += "and RTRIM({}callsign) REGEXP('{}') ".format(
-                    "sv." if count_airports_params > 0 else "", callsign
+                other_params += "and RTRIM(callsign) REGEXP('{}') ".format(
+                    callsign
                 )
+
             elif callsign.find("%") > 0 or callsign.find("_") > 0:
-                other_params += "and {}callsign ilike '{}' ".format(
-                    "sv." if count_airports_params > 0 else "", callsign
-                )
+                other_params += "and callsign ilike '{}' ".format(callsign)
             else:
-                other_params += "and {}callsign='{:<8s}' ".format(
-                    "sv." if count_airports_params > 0 else "", callsign
-                )
+                other_params += "and callsign='{:<8s}' ".format(callsign)
 
         elif isinstance(callsign, Iterable):
             callsign = ",".join("'{:<8s}'".format(c) for c in callsign)
-            other_params += "and {}callsign in ({}) ".format(
-                "sv." if count_airports_params > 0 else "", callsign
-            )
+            other_params += "and callsign in ({}) ".format(callsign)
 
         if bounds is not None:
             try:
@@ -805,11 +812,14 @@ class Impala(object):
         day_max = round_time(stop, how="after", by=timedelta(days=1))
 
         if count_airports_params > 0:
+            if isinstance(time_buffer, str):
+                time_buffer = pd.Timedelta(time_buffer)
+            buffer_s = time_buffer.total_seconds() if time_buffer else 0
             where_clause = (
-                "on sv.icao24 = est.e_icao24 and "
-                "sv.callsign = est.e_callsign and "
-                "est.firstseen <= sv.time and "
-                "sv.time <= est.lastseen "
+                "on icao24 = est.e_icao24 and "
+                "callsign = est.e_callsign and "
+                f"est.firstseen - {buffer_s} <= time and "
+                f"time <= est.lastseen + {buffer_s} "
                 "where"
             )
 
@@ -820,7 +830,7 @@ class Impala(object):
                     "either arrival_airport or departure_airport is set"
                 )
             other_tables += (
-                "as sv join (select icao24 as e_icao24, firstseen, "
+                "join (select icao24 as e_icao24, firstseen, "
                 "estdepartureairport, lastseen, estarrivalairport, "
                 "callsign as e_callsign, day from flights_data4 "
                 "where estdepartureairport ='{departure_airport}' "
@@ -834,8 +844,12 @@ class Impala(object):
             )
 
         elif arrival_airport is not None:
+            if airport is not None:
+                raise RuntimeError(
+                    "airport may not be set if " "arrival_airport is set"
+                )
             other_tables += (
-                "as sv join (select icao24 as e_icao24, firstseen, "
+                "join (select icao24 as e_icao24, firstseen, "
                 "estdepartureairport, lastseen, estarrivalairport, "
                 "callsign as e_callsign, day from flights_data4 "
                 "where estarrivalairport ='{arrival_airport}' "
@@ -847,8 +861,12 @@ class Impala(object):
             )
 
         elif departure_airport is not None:
+            if airport is not None:
+                raise RuntimeError(
+                    "airport may not be set if " "departure_airport is set"
+                )
             other_tables += (
-                "as sv join (select icao24 as e_icao24, firstseen, "
+                "join (select icao24 as e_icao24, firstseen, "
                 "estdepartureairport, lastseen, estarrivalairport, "
                 "callsign as e_callsign, day from flights_data4 "
                 "where estdepartureairport ='{departure_airport}' "
@@ -861,7 +879,7 @@ class Impala(object):
 
         elif airport is not None:
             other_tables += (
-                "as sv join (select icao24 as e_icao24, firstseen, "
+                "join (select icao24 as e_icao24, firstseen, "
                 "estdepartureairport, lastseen, estarrivalairport, "
                 "callsign as e_callsign, day from flights_data4 "
                 "where (estdepartureairport ='{arrival_or_departure_airport}' "
@@ -873,9 +891,9 @@ class Impala(object):
                 day_max=day_max.timestamp(),
             )
 
-        cumul = []
+        cumul: List[pd.DataFrame] = []
         sequence = list(split_times(start, stop, date_delta))
-        columns = ", ".join(self._impala_columns)
+        columns = ", ".join(f"{field}" for field in self._impala_columns)
         parse_columns = ", ".join(self._impala_columns)
 
         if count_airports_params > 0:
@@ -886,10 +904,8 @@ class Impala(object):
                 "estarrivalairport",
                 "day",
             ]
-            columns = (
-                ", ".join(f"sv.{field}" for field in self._impala_columns)
-                + ", "
-                + ", ".join(f"est.{field}" for field in est_columns)
+            columns = columns + (
+                ", " + ", ".join(f"est.{field}" for field in est_columns)
             )
             parse_columns = ", ".join(
                 self._impala_columns
@@ -897,10 +913,9 @@ class Impala(object):
             )
 
         if count is True:
-            other_params += "group by " + columns
+            other_params += "group by " + columns + " "
             columns = "count(*) as count, " + columns
             parse_columns = "count, " + parse_columns
-            other_tables += ", state_vectors_data4.serials s"
 
         if limit is not None:
             other_params += f"limit {limit}"
@@ -954,7 +969,7 @@ class Impala(object):
         self,
         start: timelike,
         stop: Optional[timelike] = None,
-        *args,  # more reasonable to be explicit about arguments
+        *args: Any,  # more reasonable to be explicit about arguments
         table_name: Union[None, str, List[str]] = None,
         date_delta: timedelta = timedelta(hours=1),  # noqa: B008
         icao24: Union[None, str, Iterable[str]] = None,
@@ -971,7 +986,7 @@ class Impala(object):
         other_tables: str = "",
         other_columns: Union[None, str, List[str]] = None,
         other_params: str = "",
-        progressbar: Union[bool, Callable[[Iterable], Iterable]] = True,
+        progressbar: Union[bool, ProgressbarType[Any]] = True,
     ) -> Optional[RawData]:
         """Get raw message from the OpenSky Impala shell.
 
@@ -1106,7 +1121,7 @@ class Impala(object):
         if progressbar is False:
             progressbar = iter
 
-        progressbar = cast(Callable[[Iterable], Iterable], progressbar)
+        progressbar = cast(ProgressbarType[Any], progressbar)
 
         if isinstance(icao24, str):
             other_params += f"and {table_name}.icao24='{icao24.lower()}' "
@@ -1327,7 +1342,7 @@ class Impala(object):
 
         return RawData(pd.concat(cumul).sort_values("mintime"))
 
-    def extended(self, *args, **kwargs) -> Optional[RawData]:
+    def extended(self, *args: Any, **kwargs: Any) -> Optional[RawData]:
         return self.rawdata(
             table_name="rollcall_replies_data4", *args, **kwargs
         )
