@@ -5,7 +5,7 @@ import re
 import zipfile
 from functools import reduce
 from pathlib import Path
-from typing import Dict, List, Optional, Union
+from typing import Any, Dict, List, Optional, Union
 
 from tqdm.autonotebook import tqdm
 
@@ -22,6 +22,27 @@ registration_patterns = list(
 )
 
 
+def pia_candidate(df: pd.DataFrame) -> pd.DataFrame:
+    """Returns subsets of dataframes which could be part of the PIA program.
+
+    PIA stands for Private ICAO Address.
+    Check https://doi.org/10.2514/1.I010938 for more details
+
+    - registration should be between N41000 (0xA4D691) and N42 (0xA4F946)
+    - callsign should start starts with DCM or FFL
+    - registration number in the CAR is reserved by SBS PROGRAM OFFICE.
+
+    """
+    return (
+        df.assign(_tmp=lambda df: df.icao24.apply(int, base=16))
+        .query(
+            "0xA4D691 <= _tmp < 0xA4F946 and "
+            '(callsign.str.startswith("DCM") or callsign.str.startswith("FFL"))'
+        )
+        .drop(columns=["_tmp"])
+    )
+
+
 def country(reg: Dict[str, str]) -> Dict[str, str]:
 
     # First, search the country based on the registered address intervals
@@ -77,8 +98,8 @@ class Aircraft(object):
     """
 
     `@junzis <https://github.com/junzis/>`_'s `database
-    <https://junzisun.com/adb/download>`_ is available by default in the
-    library as:
+    <https://junzisun.com/adb/download/aircraft_db.zip>`_ is available by
+    default in the library as:
 
     >>> from traffic.data import aircraft
 
@@ -103,7 +124,7 @@ class Aircraft(object):
     cache_dir: Path
     basic_columns = ["icao24", "registration", "typecode", "model", "operator"]
 
-    def __init__(self):
+    def __init__(self) -> None:
         self._junzis: Optional[pd.DataFrame] = None
         self._opensky: Optional[pd.DataFrame] = None
         self._merged: Optional[pd.DataFrame] = None
@@ -111,11 +132,11 @@ class Aircraft(object):
     def download_junzis(self) -> None:  # coverage: ignore
         """Downloads the latest version of the aircraft database by @junzis.
 
-        url: https://junzisun.com/adb/download
+        url: https://junzisun.com/adb/download/aircraft_db.zip
         """
         from .. import session
 
-        f = session.get("https://junzisun.com/adb/download")
+        f = session.get("https://junzisun.com/adb/download/aircraft_db.zip")
         with zipfile.ZipFile(io.BytesIO(f.content)) as zfile:
             with zfile.open("aircraft_db.csv", "r") as dbfile:
                 self._junzis = (
@@ -150,7 +171,7 @@ class Aircraft(object):
         assert self._junzis is not None
         return self._junzis.fillna("")
 
-    def download_opensky(self):  # coverage: ignore
+    def download_opensky(self) -> None:  # coverage: ignore
         """Downloads the latest version of the OpenSky aircraft database.
 
         url: https://opensky-network.org/aircraft-database
@@ -382,7 +403,7 @@ class Aircraft(object):
         """
         return self._fmt(self.data.query("registration.str.contains(@name)"))
 
-    def query(self, **kwargs) -> pd.DataFrame:
+    def query(self, **kwargs: Any) -> pd.DataFrame:
         """Combines several requests.
 
         The keyword arguments correspond to the name of other methods.
