@@ -1,4 +1,5 @@
 import sys
+import types
 from functools import lru_cache
 from pathlib import Path
 from typing import Union, cast
@@ -40,9 +41,11 @@ def get_flight(filename: str, directory: Path) -> Union[Flight, Traffic]:
     )
 
 
-def get_sample(module, name: str):
+def get_sample(
+    module: types.ModuleType, name: str
+) -> Union[None, "Flight", "Traffic"]:
     if sys.version_info >= (3, 7):
-        return getattr(module, name)
+        return getattr(module, name)  # type: ignore
     path = Path(module.__file__).parent
     return get_flight(name, path)
 
